
#ifndef OutputProtectionLevel_h
#define OutputProtectionLevel_h

#include "ExceptionCode.h"

#include <algorithm>
#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/Vector.h>

namespace WebCore {

class OutputProtectionLevel : public RefCounted<OutputProtectionLevel> {
public:
    static PassRefPtr<OutputProtectionLevel> create() 
    {
        return adoptRef(new OutputProtectionLevel);
    }
    static PassRefPtr<OutputProtectionLevel> create(unsigned long cDV, unsigned long uDV, unsigned long aV, unsigned long cDA, unsigned long uDA) 
    {
        return adoptRef(new OutputProtectionLevel(cDV, uDV, aV, cDA, uDA));
    }

    PassRefPtr<OutputProtectionLevel> copy();
	
	unsigned long compressedDigitalVideo() const { return m_compressedDigitalVideo; }
	unsigned long uncompressedDigitalVideo() const { return m_uncompressedDigitalVideo; }
	unsigned long analogVideo() const { return m_analogVideo; }
	unsigned long compressedDigitalAudio() const { return m_compressedDigitalAudio; }
	unsigned long uncompressedDigitalAudio() const { return m_uncompressedDigitalAudio; }

private:
 	unsigned long m_compressedDigitalVideo;
	unsigned long m_uncompressedDigitalVideo;
	unsigned long m_analogVideo;
	unsigned long m_compressedDigitalAudio;
	unsigned long m_uncompressedDigitalAudio;

	OutputProtectionLevel()
	{
		m_compressedDigitalVideo = m_uncompressedDigitalVideo = m_analogVideo = m_compressedDigitalAudio = m_uncompressedDigitalAudio = 0;
	}
	OutputProtectionLevel(unsigned long cDV, unsigned long uDV, unsigned long aV, unsigned long cDA, unsigned long uDA)
		: m_compressedDigitalVideo(cDV)
		, m_uncompressedDigitalVideo(uDV)
		, m_analogVideo(aV)
		, m_compressedDigitalAudio(cDA)
		, m_uncompressedDigitalAudio(uDA)
	{
	}
};

} // namespace WebCore

#endif
