
/*
 *  Copyright (C) 2008 INdT - Instituto Nokia de Tecnologia
 *  Copyright (C) 2009-2010 ProFUSION embedded systems
 *  Copyright (C) 2009-2010 Samsung Electronics
 *  Copyright (c) 2012 NINTENDO.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "config.h"

#if ENABLE(INSPECTOR)

#include "InspectorClientWiiU.h"
#include <wtf/Platform.h> // PLATFORM(...)

#include "Frame.h"
#include "FrameView.h"
#include "InspectorController.h"
#include "NotImplemented.h"
#include "Page.h"
#include "PlatformString.h"
#include <PlatformOS.h>
#include <shim.h>
using namespace LibPlatform;

#include "wk_event.h"
#include "WebViewWiiU.h"
#if ENABLE(INSPECTOR_SERVER)
#include "InspectorServer/WebInspectorServer.h"
#endif
using namespace WebCore;

namespace WebCore {

#if !PLATFORM(WIIU)
char InspectorClientWiiU_assetsPath[1024];

void InspectorClientWiiU::setAssetsPath(const char *set)
{
	if (set)
		strncpy(InspectorClientWiiU_assetsPath, set, sizeof(InspectorClientWiiU_assetsPath));
	else
		InspectorClientWiiU_assetsPath[0] = 0;
}

std::string InspectorClientWiiU::prependAssetsPath(const char *path)
{
	std::string fullpath = InspectorClientWiiU_assetsPath;
	if (path) fullpath.append(path);
	return fullpath;
}
#endif


namespace {

class InspectorFrontendSettingsWiiU : public WebCore::InspectorFrontendClientLocal::Settings {
public:
    virtual ~InspectorFrontendSettingsWiiU() { }
    virtual String getProperty(const String& name)
    {
        String value;

        return value;
    }

    virtual void setProperty(const String& name, const String& value)
    {

    }
};
}

InspectorClientWiiU::InspectorClientWiiU(WebView *webview, WebView *inspectorview)
    : m_inspectorView(inspectorview)
    , m_webview(webview)
#if ENABLE(INSPECTOR_SERVER)
    , m_remoteFrontendConnected(false)
    , m_remoteInspectionPageId(0)
#endif
{
#if ENABLE(INSPECTOR_SERVER)
    enableRemoteInspection();
#endif
}

void InspectorClientWiiU::inspectorDestroyed()
{
#if ENABLE(INSPECTOR_SERVER)
	if (m_remoteInspectionPageId) {
		WebInspectorServer::shared().unregisterPage(m_remoteInspectionPageId);
		WebInspectorServer::destroy();
	}
#endif
	closeInspectorFrontend();
    delete this;
}


void InspectorClientWiiU::openInspectorFrontend(InspectorController* controller)
{
	std::string url;

    if (m_inspectorView == NULL) {
        return;
    }
#if ENABLE(INSPECTOR_SERVER)
  // CRANK-ME Double check this is the right place for this
  if (m_remoteFrontendConnected) {
      return;
}
#endif
    #if PLATFORM(WIIU)
	url = "file://" + std::string(shim_GetFSContentDir()) + "/assets/inspector/inspector.html";
	#else
    #if OS(WINDOWS)
    strcpy(url, "file:///");
    #else
    strcpy(url, "file://"); // we'll be appending another leading slash anyway -TS-NSA 2012-05-16
    #endif
    strcat(url, prependAssetsPath("/inspector/inspector.html").c_str());
    #endif
    WiiULoadURLEvent *data = new WiiULoadURLEvent(m_inspectorView, url);
    WebKitSendEvent(data);
    //m_inspectorView->load(strdup(url));

    InspectorFrontendClientWiiU *frontendclient = new InspectorFrontendClientWiiU(m_webview, m_inspectorView->getPage());

    controller->setInspectorFrontendClient(adoptPtr(frontendclient));
    // Web Inspector should not belong to any other page groups since it is a specialized debugger window.
   // m_frontendWebPage->handle()->page->setGroupName("__WebInspectorPageGroup__");
}
void InspectorClientWiiU::closeInspectorFrontend()
{
  //  if (m_frontendClient)
  //      m_frontendClient->inspectorClientDestroyed();
}

void InspectorClientWiiU::bringFrontendToFront()
{
    //m_frontendClient->bringToFront();
}


void InspectorClientWiiU::highlight()
{
	//OS::Report("Inspector Client: Highlight\n");
	hideHighlight();
}

void InspectorClientWiiU::hideHighlight()
{
	//OS::Report("Inspector Client: hide Highlight\n");
	if(m_webview) {
		WebCore::Frame* frame = m_webview->getPage()->mainFrame();
		if (frame) {
			if(frame->view()) {
				LayoutRect rect = frame->view()->frameRect();
			
				if (!rect.isEmpty())
					frame->view()->invalidateRect(rect);
			}
					
		}
	}
}

void InspectorClientWiiU::populateSetting(const String&, String*)
{
	OS::Report("Inspector Client: Populate Settings\n ");
    notImplemented();
}

void InspectorClientWiiU::storeSetting(const String&, const String&)
{
	OS::Report("Inspector Client: Store Settings\n");
    notImplemented();
}

bool InspectorClientWiiU::sendMessageToFrontend(const String& message)
{
 //   OS::Report("SendMessageToFrontend: %s\n", message.utf8().data());
#if ENABLE(INSPECTOR_SERVER)
    if (m_remoteFrontendConnected) {
        sendMessageToRemoteFrontend(message);
        return true;
     }
#endif
    if (!m_inspectorView) {
        return false;
    }
    return doDispatchMessageOnFrontendPage(m_inspectorView->getPage(), message);
}

Page * InspectorClientWiiU::getInspectedPage()
{
    if (!m_webview) {
        return NULL;
    }
    return m_webview->getPage();
}

#if ENABLE(INSPECTOR_SERVER)
void InspectorClientWiiU::enableRemoteInspection()
{
    if (!m_remoteInspectionPageId)
	{
		WebInspectorServer::create();
        m_remoteInspectionPageId = WebInspectorServer::shared().registerPage(this);
	}
}

void InspectorClientWiiU::sendMessageToRemoteFrontend(const String& message)
{
	//avoid spamming
    //OS::Report("Send to Frontend: %s\n", message.utf8().data());
    ASSERT(m_remoteFrontendConnected);
    WebInspectorServer::shared().sendMessageOverConnection(m_remoteInspectionPageId, message);
}
void InspectorClientWiiU::dispatchMessageFromRemoteFrontend(const String& message)
{
	//avoid spamming
    //OS::Report("Receive from Frontend: %s\n", message.utf8().data());
	if(m_webview)
		m_webview->getPage()->inspectorController()->dispatchMessageFromFrontend(message);
}
void InspectorClientWiiU::remoteFrontendConnected()
{
    ASSERT(!m_remoteFrontendConnected);
	if(m_webview){
		m_webview->getPage()->inspectorController()->connectFrontend();
	}
	m_remoteFrontendConnected = true;
}
void InspectorClientWiiU::remoteFrontendDisconnected()
{
    ASSERT(m_remoteFrontendConnected);
	if(m_webview){
		m_webview->getPage()->inspectorController()->disconnectFrontend();
	}
	m_remoteFrontendConnected = false;
}
#endif
InspectorFrontendClientWiiU::InspectorFrontendClientWiiU(WebView *inspectedView, Page* inspectorPage)
    : InspectorFrontendClientLocal(inspectedView->getPage()->inspectorController(), inspectorPage, adoptPtr(new InspectorFrontendSettingsWiiU()))
    , m_inspectedView(inspectedView)
    , m_inspectorPage(inspectorPage)
{
}

InspectorFrontendClientWiiU::~InspectorFrontendClientWiiU()
{
}

void InspectorFrontendClientWiiU::frontendLoaded()
{
    InspectorFrontendClientLocal::frontendLoaded();
    setAttachedWindow(true);
}

String InspectorFrontendClientWiiU::localizedStringsURL()
{
    notImplemented();
    return String();
}

String InspectorFrontendClientWiiU::hiddenPanels()
{
    notImplemented();
    return String();
}

void InspectorFrontendClientWiiU::bringToFront()
{
   updateWindowTitle();
}

void InspectorFrontendClientWiiU::closeWindow()
{
    notImplemented();
}

void InspectorFrontendClientWiiU::attachWindow()
{
    notImplemented();
}

void InspectorFrontendClientWiiU::detachWindow()
{
    notImplemented();
}

void InspectorFrontendClientWiiU::setAttachedWindowHeight(unsigned)
{
    notImplemented();
}

void InspectorFrontendClientWiiU::inspectedURLChanged(const String& newURL)
{
   updateWindowTitle();
}

void InspectorFrontendClientWiiU::updateWindowTitle()
{
    notImplemented();
}

void InspectorFrontendClientWiiU::destroyInspectorView(bool notifyInspectorController)
{
    if (m_destroyingInspectorView)
        return;
    m_destroyingInspectorView = true;
    if (m_inspectedView) {
        m_inspectedView->getPage()->inspectorController()->disconnectFrontend();
    }
}

void InspectorFrontendClientWiiU::inspectorClientDestroyed()
{
    m_inspectorClient = 0;
    m_inspectedView = 0;
}

}

#endif // ENABLE(INSPECTOR)
