/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef TcpServerWiiU_h
#define TcpServerWiiU_h

#if OS(WINDOWS)
/* FIXME: CLEANUP -- because mingw-gcc has its own headers for e.g. <sys/...>,
    we have to avoid adding 'libshim/include' as a -I path, and hence need to
    spell out fuller #include paths for headers that *are* within libshim.
    There must be a better way to organize this.
   -TS-NSA 2012-10-05 */
#include <libshim/include/pthread.h>
#else
#include <pthread.h>
#endif

#include <PlatformDebugIO.h>
using namespace LibPlatform;

namespace WebCore {
class TcpServerWiiU {
public:
    enum
    {
        STATE_INIT = 0,
        STATE_LISTENING,
        STATE_CLOSE,
        STATE_BINDING,
        STATE_DISCONNECTED,
    };

    TcpServerWiiU();
    ~TcpServerWiiU();

    bool WiiUListen(unsigned long bindAddress, unsigned short port);
    void WiiUClose();
    static void HandleAccept(int fd);
    static void ResetState();

#if PLATFORM(WIIU) && defined(ENABLE_HIO_INSPECTOR)
    static void HandleAcceptHIO(int hioHandle);
#endif

private:
    static void *ClientLoop(void *data);

#if PLATFORM(WIIU) && defined(ENABLE_HIO_INSPECTOR)
    static void hio_status_callback(DebugIO::Status status, void *context);
#endif

    static int m_state;
#if PLATFORM(WIIU) || OS(UNIX)
    static int m_serverFd;
#elif OS(WINDOWS)
    static SOCKET m_serverFd;
#endif
	pthread_t m_thread;
	static pthread_mutex_t m_stateMutex;

#if PLATFORM(WIIU) && defined(ENABLE_HIO_INSPECTOR)
    DebugIO::Handle m_hioHandle;
#endif
};
}
#endif //TcpServerWiiU_h
