#include "ffmpeg_FFT_WiiU.h"
#include <libavutil/mem.h>
#include <libavcodec/avfft.h>




typedef struct _FFMPEGfftVars {
	ffmpeg_fft_mem_fn_wiiu		mpMemCallbacks;
} FFMPEGfftVars;

static FFMPEGfftVars sConfig;


typedef struct _FFMPEGfftHandle {
	RDFTContext			*mpForwardContext;
	RDFTContext			*mpInverseContext;
} FFMPEGfftHandle;




void *_ffmpeg_wiiu_malloc(size_t size)
	{
	return(sConfig.mpMemCallbacks.Allocate(size));
	}

void *_ffmpeg_wiiu_memalign(size_t align, size_t size)
	{
	return(sConfig.mpMemCallbacks.AllocateAligned(align, size));
	}

#if HAVE_POSIX_MEMALIGN
int _ffmpeg_wiiu_posix_memalign(void **ptr, size_t align, size_t size)
	{
	// Not implemented. Crash!
	*((char*)0) = 0;
	}
#endif

void *_ffmpeg_wiiu_realloc(void *ptr, size_t size)
	{
	return(sConfig.mpMemCallbacks.Reallocate(ptr, size));
	}
void _ffmpeg_wiiu_free(void *ptr)
	{
	sConfig.mpMemCallbacks.Deallocate(ptr);
	}



int _ffmpeg_FFT_Init_WiiU(const ffmpeg_fft_init_config_wiiu *pConfig)
	{
	memset(&sConfig, 0, sizeof(FFMPEGfftVars));
	sConfig.mpMemCallbacks = pConfig->mpMemCallbacks;
	return(0);
	}


void _ffmpeg_FFT_Exit_WiiU(void)
	{
	}





int _ffmpeg_FFT_Create_WiiU(void **ppHandle, int nPow2Bits)
	{
	FFMPEGfftHandle *pH;
	*ppHandle = 0;
	pH = (FFMPEGfftHandle *)av_mallocz(sizeof(FFMPEGfftHandle));
	pH->mpForwardContext = av_rdft_init(nPow2Bits, DFT_R2C);
	pH->mpInverseContext = av_rdft_init(nPow2Bits, IDFT_C2R);
	if (!pH->mpForwardContext || !pH->mpInverseContext)
		return(-1);
	*ppHandle = pH;
	return(0);
	}

int _ffmpeg_FFT_Destroy_WiiU(void *pHandle)
	{
	FFMPEGfftHandle *pH = (FFMPEGfftHandle *)pHandle;
	if (pH)
		{
		if (pH->mpInverseContext)
			av_rdft_end(pH->mpInverseContext);
		if (pH->mpForwardContext)
			av_rdft_end(pH->mpForwardContext);
		av_free(pHandle);
		}

	}

void _ffmpeg_FFT_CalcForward_WiiU(void *pHandle, float *pInOutData)
	{
	FFMPEGfftHandle *pH = (FFMPEGfftHandle *)pHandle;
	if (pH && pInOutData)
		{
		av_rdft_calc(pH->mpForwardContext, pInOutData);
		}
	}

float *_ffmpeg_FFT_CalcForward_To_WiiU(void *pHandle, const float *pInData, float *pOutData, int secondZero)
	{
	FFMPEGfftHandle *pH = (FFMPEGfftHandle *)pHandle;
	if (pH && pInData) // note: pOutData might be NULL!
		{
		return av_rdft_calc_to(pH->mpForwardContext, pInData, pOutData, secondZero);
		}
	return NULL;
	}

float *_ffmpeg_FFT_CalcInverse_To_WiiU(void *pHandle, const float *pInData, float *pOutData)
	{
	FFMPEGfftHandle *pH = (FFMPEGfftHandle *)pHandle;
	if (pH && pInData) // note: pOutData might be NULL!
		{
		return av_rdft_calc_to(pH->mpInverseContext, pInData, pOutData, 0);
		}
	return NULL;
	}

void _ffmpeg_FFT_CalcInverse_WiiU(void *pHandle, float *pInOutData)
	{
	FFMPEGfftHandle *pH = (FFMPEGfftHandle *)pHandle;
	if (pH && pInOutData)
		{
		av_rdft_calc(pH->mpInverseContext, pInOutData);
		}
	}

float *_ffmpeg_FFT_CalcInverseFromSeparate_To_WiiU(void *pHandle, const float *pInReal, const float *pInImag, float *pOutData)
	{
	FFMPEGfftHandle *pH = (FFMPEGfftHandle *)pHandle;
	if (pH && pInReal && pInImag) // note: pOutData might be NULL!
		{
		return av_rdft_calc_fromseparate_to(pH->mpInverseContext, pInReal, pInImag, pOutData);
		}
	return NULL;
	}

